# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AuthResponse,
        AuthResponseResponseData,
        CapturePaymentDetails,
        CaptureRequest,
        CaptureResponse,
        CaptureResponseData,
        GetPaidResponseData,
        InvalidTransStatusErrorType,
        PayabliApiResponseGetPaid,
        ReceiptResponse,
        RefundResponse,
        RefundWithInstructionsResponse,
        RequestCreditPaymentMethod,
        RequestPaymentAuthorizePaymentMethod,
        RequestPaymentValidatePaymentMethod,
        RequestPaymentValidatePaymentMethodMethod,
        ResponseDataRefunds,
        ReverseResponse,
        TransRequestBody,
        ValidateResponse,
        ValidateResponseData,
        VoidResponse,
        VoidResponseData,
    )
    from .errors import CaptureError, InvalidTransStatusError
_dynamic_imports: typing.Dict[str, str] = {
    "AuthResponse": ".types",
    "AuthResponseResponseData": ".types",
    "CaptureError": ".errors",
    "CapturePaymentDetails": ".types",
    "CaptureRequest": ".types",
    "CaptureResponse": ".types",
    "CaptureResponseData": ".types",
    "GetPaidResponseData": ".types",
    "InvalidTransStatusError": ".errors",
    "InvalidTransStatusErrorType": ".types",
    "PayabliApiResponseGetPaid": ".types",
    "ReceiptResponse": ".types",
    "RefundResponse": ".types",
    "RefundWithInstructionsResponse": ".types",
    "RequestCreditPaymentMethod": ".types",
    "RequestPaymentAuthorizePaymentMethod": ".types",
    "RequestPaymentValidatePaymentMethod": ".types",
    "RequestPaymentValidatePaymentMethodMethod": ".types",
    "ResponseDataRefunds": ".types",
    "ReverseResponse": ".types",
    "TransRequestBody": ".types",
    "ValidateResponse": ".types",
    "ValidateResponseData": ".types",
    "VoidResponse": ".types",
    "VoidResponseData": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AuthResponse",
    "AuthResponseResponseData",
    "CaptureError",
    "CapturePaymentDetails",
    "CaptureRequest",
    "CaptureResponse",
    "CaptureResponseData",
    "GetPaidResponseData",
    "InvalidTransStatusError",
    "InvalidTransStatusErrorType",
    "PayabliApiResponseGetPaid",
    "ReceiptResponse",
    "RefundResponse",
    "RefundWithInstructionsResponse",
    "RequestCreditPaymentMethod",
    "RequestPaymentAuthorizePaymentMethod",
    "RequestPaymentValidatePaymentMethod",
    "RequestPaymentValidatePaymentMethodMethod",
    "ResponseDataRefunds",
    "ReverseResponse",
    "TransRequestBody",
    "ValidateResponse",
    "ValidateResponseData",
    "VoidResponse",
    "VoidResponseData",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .business_section import BusinessSection
from .document_section import DocumentSection
from .owners_section import OwnersSection
from .processing_section import ProcessingSection
from .sales_section import SalesSection
from .services_section import ServicesSection
from .underwriting_data_response import UnderwritingDataResponse


class TemplateContentResponse(UniversalBaseModel):
    business_data: typing_extensions.Annotated[
        typing.Optional[BusinessSection], FieldMetadata(alias="businessData")
    ] = None
    documents_data: typing_extensions.Annotated[
        typing.Optional[DocumentSection], FieldMetadata(alias="documentsData")
    ] = None
    ownership_data: typing_extensions.Annotated[
        typing.Optional[OwnersSection], FieldMetadata(alias="ownershipData")
    ] = None
    processing_data: typing_extensions.Annotated[
        typing.Optional[ProcessingSection], FieldMetadata(alias="processingData")
    ] = None
    sales_data: typing_extensions.Annotated[typing.Optional[SalesSection], FieldMetadata(alias="salesData")] = None
    services_data: typing_extensions.Annotated[
        typing.Optional[ServicesSection], FieldMetadata(alias="servicesData")
    ] = None
    underwriting_data: typing_extensions.Annotated[
        typing.Optional[UnderwritingDataResponse], FieldMetadata(alias="underwritingData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .order import Order


class PayorFields(UniversalBaseModel):
    display: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flag indicating if the input field will show in container
    """

    fixed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flag indicating if the value in input field is read-only or not.
    """

    identifier: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flag indicating if the input field is a customer identifier
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    Label to display for field
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of field to show. Should be one of the standard customer fields or a custom field name
    """

    order: typing.Optional[Order] = None
    required: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flag indicating if the input field is required for validation
    """

    validation: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of validation to apply to the input field Accepted values:
    
      - alpha for alphabetical
    
      - numbers for numeric
    
      - text for alphanumeric
    
      - email for masked email address input
    
      - phone for US phone numbers
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    Pre-populated value for field
    """

    width: typing.Optional[int] = pydantic.Field(default=None)
    """
    Numeric value indicating the size of input relative to the container. Accepted values:
    
        - 4 = 1/3
    
        - 6 = 1/2
    
        - 8 = 2/3
    
        - 12 = 3/3
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.additional_data_string import AdditionalDataString
from ...types.bill_events import BillEvents
from ...types.bill_item import BillItem
from ...types.bill_options import BillOptions
from ...types.created_at import CreatedAt
from ...types.customer_id import CustomerId
from ...types.datenullable import Datenullable
from ...types.datetime_nullable import DatetimeNullable
from ...types.dbaname import Dbaname
from ...types.discount import Discount
from ...types.documents_ref import DocumentsRef
from ...types.duty_amount import DutyAmount
from ...types.email import Email
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.freight_amount import FreightAmount
from ...types.frequency import Frequency
from ...types.invoice_amount import InvoiceAmount
from ...types.invoice_number import InvoiceNumber
from ...types.invoice_type import InvoiceType
from ...types.invoicestatus import Invoicestatus
from ...types.org_parent_name import OrgParentName
from ...types.paylink_id import PaylinkId
from ...types.payor_data_response import PayorDataResponse
from ...types.paypoint_id import PaypointId
from ...types.purchase_order import PurchaseOrder
from ...types.shipping_from_zip import ShippingFromZip
from ...types.shippingaddress import Shippingaddress
from ...types.shippingaddressadditional import Shippingaddressadditional
from ...types.shippingcity import Shippingcity
from ...types.shippingcountry import Shippingcountry
from ...types.shippingstate import Shippingstate
from ...types.shippingzip import Shippingzip
from ...types.summary_commodity_code import SummaryCommodityCode
from ...types.tax import Tax
from ...types.terms import Terms
from ...types.terms_conditions import TermsConditions
from .invoice_id import InvoiceId
from .invoice_paid_amount import InvoicePaidAmount


class GetInvoiceRecord(UniversalBaseModel):
    invoice_id: typing_extensions.Annotated[InvoiceId, FieldMetadata(alias="invoiceId")]
    customer_id: typing_extensions.Annotated[CustomerId, FieldMetadata(alias="customerId")]
    paypoint_id: typing_extensions.Annotated[PaypointId, FieldMetadata(alias="paypointId")]
    invoice_number: typing_extensions.Annotated[InvoiceNumber, FieldMetadata(alias="invoiceNumber")]
    invoice_date: typing_extensions.Annotated[Datenullable, FieldMetadata(alias="invoiceDate")]
    invoice_due_date: typing_extensions.Annotated[Datenullable, FieldMetadata(alias="invoiceDueDate")]
    invoice_sent_date: typing_extensions.Annotated[DatetimeNullable, FieldMetadata(alias="invoiceSentDate")]
    invoice_end_date: typing_extensions.Annotated[Datenullable, FieldMetadata(alias="invoiceEndDate")]
    last_payment_date: typing_extensions.Annotated[DatetimeNullable, FieldMetadata(alias="lastPaymentDate")]
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")]
    invoice_status: typing_extensions.Annotated[Invoicestatus, FieldMetadata(alias="invoiceStatus")]
    invoice_type: typing_extensions.Annotated[InvoiceType, FieldMetadata(alias="invoiceType")]
    frequency: Frequency
    payment_terms: typing_extensions.Annotated[Terms, FieldMetadata(alias="paymentTerms")]
    terms_conditions: typing_extensions.Annotated[
        typing.Optional[TermsConditions], FieldMetadata(alias="termsConditions")
    ] = None
    notes: typing.Optional[str] = None
    tax: Tax
    discount: Discount
    invoice_amount: typing_extensions.Annotated[InvoiceAmount, FieldMetadata(alias="invoiceAmount")]
    invoice_paid_amount: typing_extensions.Annotated[InvoicePaidAmount, FieldMetadata(alias="invoicePaidAmount")]
    freight_amount: typing_extensions.Annotated[FreightAmount, FieldMetadata(alias="freightAmount")]
    duty_amount: typing_extensions.Annotated[DutyAmount, FieldMetadata(alias="dutyAmount")]
    purchase_order: typing_extensions.Annotated[PurchaseOrder, FieldMetadata(alias="purchaseOrder")]
    first_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="firstName")] = pydantic.Field(
        default=None
    )
    """
    First name of the recipient of the invoice.
    """

    last_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lastName")] = pydantic.Field(
        default=None
    )
    """
    Last name of the recipient of the invoice.
    """

    company: typing.Optional[str] = pydantic.Field(default=None)
    """
    Company name of the recipient of the invoice.
    """

    shipping_address_1: typing_extensions.Annotated[Shippingaddress, FieldMetadata(alias="shippingAddress1")]
    shipping_address_2: typing_extensions.Annotated[Shippingaddressadditional, FieldMetadata(alias="shippingAddress2")]
    shipping_city: typing_extensions.Annotated[Shippingcity, FieldMetadata(alias="shippingCity")]
    shipping_state: typing_extensions.Annotated[Shippingstate, FieldMetadata(alias="shippingState")]
    shipping_zip: typing_extensions.Annotated[Shippingzip, FieldMetadata(alias="shippingZip")]
    shipping_from_zip: typing_extensions.Annotated[ShippingFromZip, FieldMetadata(alias="shippingFromZip")]
    shipping_country: typing_extensions.Annotated[Shippingcountry, FieldMetadata(alias="shippingCountry")]
    shipping_email: typing_extensions.Annotated[Email, FieldMetadata(alias="shippingEmail")]
    shipping_phone: typing_extensions.Annotated[str, FieldMetadata(alias="shippingPhone")]
    summary_commodity_code: typing_extensions.Annotated[
        SummaryCommodityCode, FieldMetadata(alias="summaryCommodityCode")
    ]
    items: typing.List[BillItem]
    customer: typing_extensions.Annotated[PayorDataResponse, FieldMetadata(alias="Customer")]
    paylink_id: typing_extensions.Annotated[PaylinkId, FieldMetadata(alias="paylinkId")]
    bill_events: typing_extensions.Annotated[BillEvents, FieldMetadata(alias="billEvents")]
    scheduled_options: typing_extensions.Annotated[BillOptions, FieldMetadata(alias="scheduledOptions")]
    paypoint_legalname: typing_extensions.Annotated[str, FieldMetadata(alias="PaypointLegalname")]
    paypoint_dbaname: typing_extensions.Annotated[Dbaname, FieldMetadata(alias="PaypointDbaname")]
    paypoint_entryname: typing_extensions.Annotated[Entrypointfield, FieldMetadata(alias="PaypointEntryname")]
    parent_org_name: typing_extensions.Annotated[OrgParentName, FieldMetadata(alias="ParentOrgName")]
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="AdditionalData")
    ] = None
    documents_ref: typing_extensions.Annotated[DocumentsRef, FieldMetadata(alias="DocumentsRef")]
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.page_identifier import PageIdentifier
from ...types.pagesize import Pagesize
from ...types.totalpages import Totalpages
from ...types.totalrecords import Totalrecords


class BatchDetailResponseSummary(UniversalBaseModel):
    service_fees: typing_extensions.Annotated[float, FieldMetadata(alias="serviceFees")]
    transfer_amount: typing_extensions.Annotated[float, FieldMetadata(alias="transferAmount")]
    refunds: float
    held_amount: typing_extensions.Annotated[float, FieldMetadata(alias="heldAmount")]
    total_records: typing_extensions.Annotated[Totalrecords, FieldMetadata(alias="totalRecords")]
    total_amount: typing_extensions.Annotated[float, FieldMetadata(alias="totalAmount")]
    total_net_amount: typing_extensions.Annotated[float, FieldMetadata(alias="totalNetAmount")]
    total_pages: typing_extensions.Annotated[Totalpages, FieldMetadata(alias="totalPages")]
    page_size: typing_extensions.Annotated[Pagesize, FieldMetadata(alias="pageSize")]
    pageidentifier: typing.Optional[PageIdentifier] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.bill_data import BillData
from ..types.entrypointfield import Entrypointfield
from ..types.force_customer_creation import ForceCustomerCreation
from ..types.idempotency_key import IdempotencyKey
from ..types.payment_detail import PaymentDetail
from ..types.payor_data_request import PayorDataRequest
from ..types.schedule_detail import ScheduleDetail
from ..types.source import Source
from ..types.subdomain import Subdomain
from ..types.subscription_query_records import SubscriptionQueryRecords
from .raw_client import AsyncRawSubscriptionClient, RawSubscriptionClient
from .types.add_subscription_response import AddSubscriptionResponse
from .types.remove_subscription_response import RemoveSubscriptionResponse
from .types.request_schedule_payment_method import RequestSchedulePaymentMethod
from .types.set_pause import SetPause
from .types.update_subscription_response import UpdateSubscriptionResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SubscriptionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSubscriptionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSubscriptionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSubscriptionClient
        """
        return self._raw_client

    def get_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionQueryRecords:
        """
        Retrieves a single subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionQueryRecords
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.subscription.get_subscription(
            sub_id=263,
        )
        """
        _response = self._raw_client.get_subscription(sub_id, request_options=request_options)
        return _response.data

    def new_subscription(
        self,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        payment_method: typing.Optional[RequestSchedulePaymentMethod] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddSubscriptionResponse:
        """
        Creates a subscription or scheduled payment to run at a specified time and frequency.

        Parameters
        ----------
        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor.

        entry_point : typing.Optional[Entrypointfield]

        invoice_data : typing.Optional[BillData]
            Object describing an Invoice linked to the subscription.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        payment_method : typing.Optional[RequestSchedulePaymentMethod]
            Information about the payment method for the transaction. Required and recommended fields for each payment method type are described in each schema below.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription.

        set_pause : typing.Optional[SetPause]

        source : typing.Optional[Source]

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddSubscriptionResponse
            Success

        Examples
        --------
        from payabli import (
            PaymentDetail,
            PayMethodCredit,
            PayorDataRequest,
            ScheduleDetail,
            payabli,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.subscription.new_subscription(
            customer_data=PayorDataRequest(
                customer_id=4440,
            ),
            entry_point="f743aed24a",
            payment_details=PaymentDetail(
                service_fee=0.0,
                total_amount=100.0,
            ),
            payment_method=PayMethodCredit(
                cardcvv="123",
                cardexp="02/25",
                card_holder="John Cassian",
                cardnumber="4111111111111111",
                cardzip="37615",
                initiator="payor",
            ),
            schedule_details=ScheduleDetail(
                end_date="03-20-2025",
                frequency="weekly",
                plan_id=1,
                start_date="09-20-2024",
            ),
        )
        """
        _response = self._raw_client.new_subscription(
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            entry_point=entry_point,
            invoice_data=invoice_data,
            payment_details=payment_details,
            payment_method=payment_method,
            schedule_details=schedule_details,
            set_pause=set_pause,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data

    def remove_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveSubscriptionResponse:
        """
        Deletes a subscription, autopay, or recurring payment and prevents future charges.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveSubscriptionResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.subscription.remove_subscription(
            sub_id=396,
        )
        """
        _response = self._raw_client.remove_subscription(sub_id, request_options=request_options)
        return _response.data

    def update_subscription(
        self,
        sub_id: int,
        *,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSubscriptionResponse:
        """
        Updates a subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription

        set_pause : typing.Optional[SetPause]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSubscriptionResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.subscription.update_subscription(
            sub_id=231,
            set_pause=True,
        )
        """
        _response = self._raw_client.update_subscription(
            sub_id,
            payment_details=payment_details,
            schedule_details=schedule_details,
            set_pause=set_pause,
            request_options=request_options,
        )
        return _response.data


class AsyncSubscriptionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSubscriptionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSubscriptionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSubscriptionClient
        """
        return self._raw_client

    async def get_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionQueryRecords:
        """
        Retrieves a single subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionQueryRecords
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscription.get_subscription(
                sub_id=263,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_subscription(sub_id, request_options=request_options)
        return _response.data

    async def new_subscription(
        self,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        payment_method: typing.Optional[RequestSchedulePaymentMethod] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddSubscriptionResponse:
        """
        Creates a subscription or scheduled payment to run at a specified time and frequency.

        Parameters
        ----------
        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor.

        entry_point : typing.Optional[Entrypointfield]

        invoice_data : typing.Optional[BillData]
            Object describing an Invoice linked to the subscription.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        payment_method : typing.Optional[RequestSchedulePaymentMethod]
            Information about the payment method for the transaction. Required and recommended fields for each payment method type are described in each schema below.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription.

        set_pause : typing.Optional[SetPause]

        source : typing.Optional[Source]

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import (
            Asyncpayabli,
            PaymentDetail,
            PayMethodCredit,
            PayorDataRequest,
            ScheduleDetail,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscription.new_subscription(
                customer_data=PayorDataRequest(
                    customer_id=4440,
                ),
                entry_point="f743aed24a",
                payment_details=PaymentDetail(
                    service_fee=0.0,
                    total_amount=100.0,
                ),
                payment_method=PayMethodCredit(
                    cardcvv="123",
                    cardexp="02/25",
                    card_holder="John Cassian",
                    cardnumber="4111111111111111",
                    cardzip="37615",
                    initiator="payor",
                ),
                schedule_details=ScheduleDetail(
                    end_date="03-20-2025",
                    frequency="weekly",
                    plan_id=1,
                    start_date="09-20-2024",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.new_subscription(
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            entry_point=entry_point,
            invoice_data=invoice_data,
            payment_details=payment_details,
            payment_method=payment_method,
            schedule_details=schedule_details,
            set_pause=set_pause,
            source=source,
            subdomain=subdomain,
            request_options=request_options,
        )
        return _response.data

    async def remove_subscription(
        self, sub_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveSubscriptionResponse:
        """
        Deletes a subscription, autopay, or recurring payment and prevents future charges.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscription.remove_subscription(
                sub_id=396,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_subscription(sub_id, request_options=request_options)
        return _response.data

    async def update_subscription(
        self,
        sub_id: int,
        *,
        payment_details: typing.Optional[PaymentDetail] = OMIT,
        schedule_details: typing.Optional[ScheduleDetail] = OMIT,
        set_pause: typing.Optional[SetPause] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSubscriptionResponse:
        """
        Updates a subscription's details.

        Parameters
        ----------
        sub_id : int
            The subscription ID.

        payment_details : typing.Optional[PaymentDetail]
            Object describing details of the payment. To skip the payment, set the `totalAmount` to 0. Payments will be paused until the amount is updated to a non-zero value. When `totalAmount` is set to 0, the `serviceFee` must also be set to 0.

        schedule_details : typing.Optional[ScheduleDetail]
            Object describing the schedule for subscription

        set_pause : typing.Optional[SetPause]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSubscriptionResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscription.update_subscription(
                sub_id=231,
                set_pause=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_subscription(
            sub_id,
            payment_details=payment_details,
            schedule_details=schedule_details,
            set_pause=set_pause,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.account_number import AccountNumber
from ...types.bank_account_holder_name import BankAccountHolderName
from ...types.bank_name import BankName
from ...types.routing_account import RoutingAccount
from ...types.type_account import TypeAccount


class RequestOutAuthorizeVendorBillingData(UniversalBaseModel):
    """
    Object containing vendor's bank information. This object is deprecated for this endpoint. Use the `paymentMethod` object in payout authorize requests instead.
    """

    bank_name: typing_extensions.Annotated[typing.Optional[BankName], FieldMetadata(alias="bankName")] = None
    routing_account: typing_extensions.Annotated[
        typing.Optional[RoutingAccount], FieldMetadata(alias="routingAccount")
    ] = None
    account_number: typing_extensions.Annotated[
        typing.Optional[AccountNumber], FieldMetadata(alias="accountNumber")
    ] = None
    type_account: typing_extensions.Annotated[typing.Optional[TypeAccount], FieldMetadata(alias="typeAccount")] = None
    bank_account_holder_name: typing_extensions.Annotated[
        typing.Optional[BankAccountHolderName], FieldMetadata(alias="bankAccountHolderName")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accounting_field import AccountingField
from ...types.additional_data_string import AdditionalDataString
from ...types.bill_approvals import BillApprovals
from ...types.bill_events import BillEvents
from ...types.bill_id import BillId
from ...types.billitems import Billitems
from ...types.billstatus import Billstatus
from ...types.comments import Comments
from ...types.created_at import CreatedAt
from ...types.datenullable import Datenullable
from ...types.dbaname import Dbaname
from ...types.documents_ref import DocumentsRef
from ...types.entity_id import EntityId
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.frequency import Frequency
from ...types.last_modified import LastModified
from ...types.legalname import Legalname
from ...types.org_parent_id import OrgParentId
from ...types.org_parent_name import OrgParentName
from ...types.paylink_id import PaylinkId
from ...types.terms import Terms
from ...types.transaction_out_query_record import TransactionOutQueryRecord
from ...types.vendor_data_response import VendorDataResponse


class BillResponseData(UniversalBaseModel):
    id_bill: typing_extensions.Annotated[typing.Optional[BillId], FieldMetadata(alias="IdBill")] = None
    bill_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BillNumber")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier for the bill.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="NetAmount")] = pydantic.Field(
        default=None
    )
    """
    Net amount owed in bill.
    """

    discount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="Discount")] = pydantic.Field(
        default=None
    )
    """
    Bill discount amount.
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount for the bill.
    """

    bill_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="BillDate")] = (
        pydantic.Field(default=None)
    )
    """
    Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY
    """

    due_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="DueDate")] = (
        pydantic.Field(default=None)
    )
    """
    Due Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY
    """

    comments: typing_extensions.Annotated[typing.Optional[Comments], FieldMetadata(alias="Comments")] = None
    batch_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BatchNumber")] = (
        pydantic.Field(default=None)
    )
    """
    The batch number that the bill belongs to.
    """

    bill_items: typing_extensions.Annotated[typing.Optional[Billitems], FieldMetadata(alias="BillItems")] = (
        pydantic.Field(default=None)
    )
    """
    Array of `LineItems` contained in bill.
    """

    mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Mode")] = pydantic.Field(default=None)
    """
    Bill mode: value `0` for single/one-time bills, `1` for scheduled bills.
    """

    payment_method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentMethod")] = (
        pydantic.Field(default=None)
    )
    """
    Payment method used for the bill.
    """

    payment_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaymentId")] = pydantic.Field(
        default=None
    )
    """
    Payment ID associated with the bill.
    """

    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="AccountingField1")
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="AccountingField2")
    ] = None
    terms: typing_extensions.Annotated[typing.Optional[Terms], FieldMetadata(alias="Terms")] = None
    source: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Source")] = pydantic.Field(
        default=None
    )
    """
    The source of the bill, such as "API" or "UI".
    """

    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="AdditionalData")
    ] = None
    vendor: typing_extensions.Annotated[typing.Optional[VendorDataResponse], FieldMetadata(alias="Vendor")] = None
    status: typing_extensions.Annotated[typing.Optional[Billstatus], FieldMetadata(alias="Status")] = None
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt")] = None
    end_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="EndDate")] = (
        pydantic.Field(default=None)
    )
    """
    End date for scheduled bills. Applied only in `Mode` = 1.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="LastUpdated")] = None
    frequency: typing_extensions.Annotated[typing.Optional[Frequency], FieldMetadata(alias="Frequency")] = (
        pydantic.Field(default=None)
    )
    """
    Frequency for scheduled bills. Applied only in `Mode` = 1.
    """

    transaction: typing_extensions.Annotated[
        typing.Optional[TransactionOutQueryRecord], FieldMetadata(alias="Transaction")
    ] = pydantic.Field(default=None)
    """
    MoneyOut transaction associated to the bill
    """

    bill_events: typing_extensions.Annotated[typing.Optional[BillEvents], FieldMetadata(alias="billEvents")] = None
    bill_approvals: typing_extensions.Annotated[
        typing.Optional[BillApprovals], FieldMetadata(alias="billApprovals")
    ] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        None
    )
    parent_org_id: typing_extensions.Annotated[typing.Optional[OrgParentId], FieldMetadata(alias="ParentOrgId")] = None
    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = None
    paylink_id: typing_extensions.Annotated[typing.Optional[PaylinkId], FieldMetadata(alias="paylinkId")] = None
    documents_ref: typing_extensions.Annotated[typing.Optional[DocumentsRef], FieldMetadata(alias="DocumentsRef")] = (
        pydantic.Field(default=None)
    )
    """
    Object with the attached documents.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    lot_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="LotNumber")] = pydantic.Field(
        default=None
    )
    """
    Lot number of the bill.
    """

    entity_id: typing_extensions.Annotated[typing.Optional[EntityId], FieldMetadata(alias="EntityID")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TierItemPass(UniversalBaseModel):
    amount_feeone_time: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="amountFeeone-time")
    ] = None
    amount_fee_recurring: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="amountFeeRecurring")
    ] = None
    high_pay_range: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="highPayRange")] = None
    low_pay_range: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="lowPayRange")] = None
    percent_feeone_time: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="percentFeeone-time")
    ] = None
    percent_fee_recurring: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="percentFeeRecurring")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .enabled import Enabled
from .order import Order


class NoteElement(UniversalBaseModel):
    enabled: typing.Optional[Enabled] = None
    header: typing.Optional[str] = pydantic.Field(default=None)
    """
    Header text for section
    """

    order: typing.Optional[Order] = None
    placeholder: typing.Optional[str] = pydantic.Field(default=None)
    """
    Placeholder text for input field
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    Pre-populated value for input field
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .authcode import Authcode
from .avsresponsetext import Avsresponsetext
from .customeridtrans import Customeridtrans
from .cvvresponsetext import Cvvresponsetext
from .method_reference_id import MethodReferenceId
from .referenceidtrans import Referenceidtrans
from .result_code import ResultCode
from .resulttext import Resulttext


class PayabliApiResponse0ResponseData(UniversalBaseModel):
    auth_code: typing_extensions.Annotated[typing.Optional[Authcode], FieldMetadata(alias="AuthCode")] = None
    avs_response_text: typing_extensions.Annotated[
        typing.Optional[Avsresponsetext], FieldMetadata(alias="avsResponseText")
    ] = None
    customer_id: typing_extensions.Annotated[typing.Optional[Customeridtrans], FieldMetadata(alias="CustomerId")] = None
    cvv_response_text: typing_extensions.Annotated[
        typing.Optional[Cvvresponsetext], FieldMetadata(alias="cvvResponseText")
    ] = None
    method_reference_id: typing_extensions.Annotated[
        typing.Optional[MethodReferenceId], FieldMetadata(alias="methodReferenceId")
    ] = None
    reference_id: typing_extensions.Annotated[typing.Optional[Referenceidtrans], FieldMetadata(alias="ReferenceId")] = (
        None
    )
    result_code: typing_extensions.Annotated[typing.Optional[ResultCode], FieldMetadata(alias="ResultCode")] = None
    result_text: typing_extensions.Annotated[typing.Optional[Resulttext], FieldMetadata(alias="ResultText")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_types_tiers import AchTypesTiers
from .visible import Visible


class AchAbsorbSection(UniversalBaseModel):
    multi_tier: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="multiTier")] = None
    tiers: typing.Optional[typing.List[AchTypesTiers]] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.vendor_check_number import VendorCheckNumber


class RequestOutAuthorizePaymentDetails(UniversalBaseModel):
    """
    Object containing payment details.
    """

    check_number: typing_extensions.Annotated[
        typing.Optional[VendorCheckNumber], FieldMetadata(alias="checkNumber")
    ] = None
    currency: typing.Optional[str] = pydantic.Field(default=None)
    """
    Currency code ISO-4217. If not code is provided the currency in the paypoint setting is taken. Default is **USD**.
    """

    service_fee: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="serviceFee")] = (
        pydantic.Field(default=None)
    )
    """
    Service fee to be deducted from the total amount. This amount must be a number, percentages aren't accepted. If you are using a percentage-based fee schedule, you must calculate the value manually.
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount to be charged. If a service fee is included, then this amount should include the service fee.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accept_oauth import AcceptOauth
from .accept_register import AcceptRegister
from .entry_attributes import EntryAttributes
from .last_modified import LastModified
from .org_parent_name import OrgParentName
from .reference_name import ReferenceName
from .reference_template_id import ReferenceTemplateId
from .template_code import TemplateCode
from .template_name import TemplateName


class QueryBoardingLinksResponseRecordsItem(UniversalBaseModel):
    accept_oauth: typing_extensions.Annotated[typing.Optional[AcceptOauth], FieldMetadata(alias="AcceptOauth")] = None
    accept_register: typing_extensions.Annotated[
        typing.Optional[AcceptRegister], FieldMetadata(alias="AcceptRegister")
    ] = None
    entry_attributes: typing_extensions.Annotated[
        typing.Optional[EntryAttributes], FieldMetadata(alias="EntryAttributes")
    ] = None
    id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Id")] = pydantic.Field(default=None)
    """
    The record ID.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="LastUpdated")] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="OrgParentName")
    ] = None
    reference_name: typing_extensions.Annotated[
        typing.Optional[ReferenceName], FieldMetadata(alias="ReferenceName")
    ] = None
    reference_template_id: typing_extensions.Annotated[
        typing.Optional[ReferenceTemplateId], FieldMetadata(alias="ReferenceTemplateId")
    ] = None
    template_code: typing_extensions.Annotated[typing.Optional[TemplateCode], FieldMetadata(alias="TemplateCode")] = (
        None
    )
    template_name: typing_extensions.Annotated[typing.Optional[TemplateName], FieldMetadata(alias="TemplateName")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.cloud_query_api_response import CloudQueryApiResponse
from ..types.idempotency_key import IdempotencyKey
from .raw_client import AsyncRawCloudClient, RawCloudClient
from .types.add_device_response import AddDeviceResponse
from .types.remove_device_response import RemoveDeviceResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CloudClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCloudClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCloudClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCloudClient
        """
        return self._raw_client

    def add_device(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        description: typing.Optional[str] = OMIT,
        registration_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddDeviceResponse:
        """
        Register a cloud device to an entrypoint. See [Devices Quickstart](/developers/developer-guides/devices-quickstart#devices-quickstart) for a complete guide.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        description : typing.Optional[str]
            Description or name for the device. This can be anything, but Payabli recommends entering the name of the paypoint, or some other easy to identify descriptor. If you have several devices for one paypoint, you can give them descriptions like "Cashier 1" and "Cashier 2", or "Front Desk" and "Back Office"

        registration_code : typing.Optional[str]
            The device registration code or serial number, depending on the model.

            - Ingenico devices: This is the activation code that's displayed on the device screen during setup.

            - PAX A920 device: This code is the serial number on the back of the device.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddDeviceResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.cloud.add_device(
            entry="8cfec329267",
            registration_code="YS7DS5",
            description="Front Desk POS",
        )
        """
        _response = self._raw_client.add_device(
            entry,
            idempotency_key=idempotency_key,
            description=description,
            registration_code=registration_code,
            request_options=request_options,
        )
        return _response.data

    def history_device(
        self, device_id: str, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CloudQueryApiResponse:
        """
        Retrieve the registration history for a device.

        Parameters
        ----------
        device_id : str
            ID of the cloud device.

        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloudQueryApiResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.cloud.history_device(
            device_id="WXGDWB",
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.history_device(device_id, entry, request_options=request_options)
        return _response.data

    def list_device(
        self,
        entry: str,
        *,
        force_refresh: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloudQueryApiResponse:
        """
        Get a list of cloud devices registered to an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        force_refresh : typing.Optional[bool]
            When `true`, the request retrieves an updated list of devices from the processor instead of returning a cached list of devices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloudQueryApiResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.cloud.list_device(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.list_device(entry, force_refresh=force_refresh, request_options=request_options)
        return _response.data

    def remove_device(
        self, device_id: str, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveDeviceResponse:
        """
        Remove a cloud device from an entrypoint.

        Parameters
        ----------
        device_id : str
            ID of the cloud device.

        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveDeviceResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.cloud.remove_device(
            device_id="6c361c7d-674c-44cc-b790-382b75d1xxx",
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.remove_device(device_id, entry, request_options=request_options)
        return _response.data


class AsyncCloudClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCloudClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCloudClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCloudClient
        """
        return self._raw_client

    async def add_device(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        description: typing.Optional[str] = OMIT,
        registration_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddDeviceResponse:
        """
        Register a cloud device to an entrypoint. See [Devices Quickstart](/developers/developer-guides/devices-quickstart#devices-quickstart) for a complete guide.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        description : typing.Optional[str]
            Description or name for the device. This can be anything, but Payabli recommends entering the name of the paypoint, or some other easy to identify descriptor. If you have several devices for one paypoint, you can give them descriptions like "Cashier 1" and "Cashier 2", or "Front Desk" and "Back Office"

        registration_code : typing.Optional[str]
            The device registration code or serial number, depending on the model.

            - Ingenico devices: This is the activation code that's displayed on the device screen during setup.

            - PAX A920 device: This code is the serial number on the back of the device.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddDeviceResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cloud.add_device(
                entry="8cfec329267",
                registration_code="YS7DS5",
                description="Front Desk POS",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_device(
            entry,
            idempotency_key=idempotency_key,
            description=description,
            registration_code=registration_code,
            request_options=request_options,
        )
        return _response.data

    async def history_device(
        self, device_id: str, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CloudQueryApiResponse:
        """
        Retrieve the registration history for a device.

        Parameters
        ----------
        device_id : str
            ID of the cloud device.

        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloudQueryApiResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cloud.history_device(
                device_id="WXGDWB",
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.history_device(device_id, entry, request_options=request_options)
        return _response.data

    async def list_device(
        self,
        entry: str,
        *,
        force_refresh: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloudQueryApiResponse:
        """
        Get a list of cloud devices registered to an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        force_refresh : typing.Optional[bool]
            When `true`, the request retrieves an updated list of devices from the processor instead of returning a cached list of devices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloudQueryApiResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cloud.list_device(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_device(
            entry, force_refresh=force_refresh, request_options=request_options
        )
        return _response.data

    async def remove_device(
        self, device_id: str, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveDeviceResponse:
        """
        Remove a cloud device from an entrypoint.

        Parameters
        ----------
        device_id : str
            ID of the cloud device.

        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveDeviceResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.cloud.remove_device(
                device_id="6c361c7d-674c-44cc-b790-382b75d1xxx",
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_device(device_id, entry, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .batch_number import BatchNumber
from .dbaname import Dbaname
from .general_events import GeneralEvents
from .legalname import Legalname
from .paypoint_id import PaypointId
from .transfer_bank_account import TransferBankAccount
from .transfer_identifier import TransferIdentifier
from .transfer_message import TransferMessage


class Transfer(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import GeneralEvents, Transfer, TransferBankAccount

    Transfer(
        transfer_id=79851,
        paypoint_id=705,
        batch_number="split_705_gp_11-16-2024",
        batch_currency="USD",
        batch_records=1,
        transfer_identifier="bbcbfed7-e535-45fe-8d62-000000",
        batch_id=111430,
        paypoint_entry_name="47ae3de37",
        paypoint_legal_name="Gruzya Outdoor Outfitters LLC",
        paypoint_dba_name="Gruzya Outdoor Outfitters",
        paypoint_logo="https://example.com/logo.png",
        parent_org_name="Pilgrim Planner",
        parent_org_id=12345,
        parent_org_logo="https://example.com/parent-logo.png",
        external_paypoint_id="ext-12345",
        bank_account=TransferBankAccount(
            account_number="****1234",
            routing_number="123456789",
        ),
        transfer_date="2024-11-17T08:20:07.288+00:00",
        processor="gp",
        transfer_status=2,
        gross_amount=1029.0,
        charge_back_amount=25.0,
        returned_amount=0.0,
        hold_amount=0.0,
        released_amount=0.0,
        billing_fees_amount=0.0,
        third_party_paid_amount=0.0,
        adjustments_amount=0.0,
        net_transfer_amount=1004.0,
        events_data=[
            GeneralEvents(
                description="Transfer Created",
                event_time=datetime.datetime.fromisoformat(
                    "2024-11-16 08:15:33.436000+00:00",
                ),
                source="worker",
            )
        ],
        messages=[],
    )
    """

    transfer_id: typing_extensions.Annotated[int, FieldMetadata(alias="transferId")] = pydantic.Field()
    """
    The transfer ID.
    """

    paypoint_id: typing_extensions.Annotated[PaypointId, FieldMetadata(alias="paypointId")]
    batch_number: typing_extensions.Annotated[BatchNumber, FieldMetadata(alias="batchNumber")]
    batch_currency: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="batchCurrency")] = (
        pydantic.Field(default=None)
    )
    """
    The currency of the batch, either USD or CAD.
    """

    batch_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="batchRecords")] = (
        pydantic.Field(default=None)
    )
    """
    Number of records in the batch.
    """

    transfer_identifier: typing_extensions.Annotated[TransferIdentifier, FieldMetadata(alias="transferIdentifier")]
    batch_id: typing_extensions.Annotated[int, FieldMetadata(alias="batchId")] = pydantic.Field()
    """
    The ID of the batch the transfer belongs to.
    """

    paypoint_entry_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paypointEntryName")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint entry name.
    """

    paypoint_legal_name: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="paypointLegalName")
    ] = pydantic.Field(default=None)
    """
    The paypoint legal name.
    """

    paypoint_dba_name: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="paypointDbaName")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint DBA name.
    """

    paypoint_logo: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paypointLogo")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint logo URL.
    """

    parent_org_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentOrgName")] = (
        pydantic.Field(default=None)
    )
    """
    The parent organization name.
    """

    parent_org_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="parentOrgId")] = (
        pydantic.Field(default=None)
    )
    """
    The parent organization ID.
    """

    parent_org_logo: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentOrgLogo")] = (
        pydantic.Field(default=None)
    )
    """
    The parent organization logo URL.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="externalPaypointId")
    ] = pydantic.Field(default=None)
    """
    The external paypoint ID.
    """

    bank_account: typing_extensions.Annotated[
        typing.Optional[TransferBankAccount], FieldMetadata(alias="bankAccount")
    ] = pydantic.Field(default=None)
    """
    Bank account information for the transfer.
    """

    transfer_date: typing_extensions.Annotated[str, FieldMetadata(alias="transferDate")] = pydantic.Field()
    """
    Date when the transfer occurred.
    """

    processor: str = pydantic.Field()
    """
    The payment processor used for the transfer.
    """

    transfer_status: typing_extensions.Annotated[int, FieldMetadata(alias="transferStatus")] = pydantic.Field()
    """
    The current status of the transfer.
    """

    gross_amount: typing_extensions.Annotated[float, FieldMetadata(alias="grossAmount")] = pydantic.Field()
    """
    Gross batch is the total amount of the payments grouped in the batch. This amount includes service fees.
    """

    charge_back_amount: typing_extensions.Annotated[float, FieldMetadata(alias="chargeBackAmount")] = pydantic.Field()
    """
    Amount of chargebacks to be deducted from batch.
    """

    returned_amount: typing_extensions.Annotated[float, FieldMetadata(alias="returnedAmount")] = pydantic.Field()
    """
    Amount of ACH returns to be deducted from batch.
    """

    hold_amount: typing_extensions.Annotated[float, FieldMetadata(alias="holdAmount")] = pydantic.Field()
    """
    Amount being held for fraud or risk concerns.
    """

    released_amount: typing_extensions.Annotated[float, FieldMetadata(alias="releasedAmount")] = pydantic.Field()
    """
    Amount of previously held funds that have been released after a risk review.
    """

    billing_fees_amount: typing_extensions.Annotated[float, FieldMetadata(alias="billingFeesAmount")] = pydantic.Field()
    """
    Amount of charges and fees applied for services and transactions.
    """

    third_party_paid_amount: typing_extensions.Annotated[float, FieldMetadata(alias="thirdPartyPaidAmount")] = (
        pydantic.Field()
    )
    """
    Amount of payments captured in the batch cycle that are deposited separately. For example, checks or cash payments recorded in the batch but not deposited via Payabli, or card brands making a direct transfer in certain situations.
    """

    adjustments_amount: typing_extensions.Annotated[float, FieldMetadata(alias="adjustmentsAmount")] = pydantic.Field()
    """
    Amount of corrections applied to Billing & Fees charges.
    """

    net_transfer_amount: typing_extensions.Annotated[float, FieldMetadata(alias="netTransferAmount")] = pydantic.Field()
    """
    The net transfer amount after all deductions and additions.
    """

    events_data: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]], FieldMetadata(alias="eventsData")
    ] = pydantic.Field(default=None)
    """
    List of events associated with the transfer.
    """

    messages: typing.Optional[typing.List[TransferMessage]] = pydantic.Field(default=None)
    """
    List of messages related to the transfer.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .link_data import LinkData


class OList(UniversalBaseModel):
    oaddress: typing.Optional[LinkData] = None
    ocity: typing.Optional[LinkData] = None
    ocountry: typing.Optional[LinkData] = None
    odriverstate: typing.Optional[LinkData] = None
    ostate: typing.Optional[LinkData] = None
    ownerdob: typing.Optional[LinkData] = None
    ownerdriver: typing.Optional[LinkData] = None
    owneremail: typing.Optional[LinkData] = None
    ownername: typing.Optional[LinkData] = None
    ownerpercent: typing.Optional[LinkData] = None
    ownerphone_1: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="ownerphone1")] = None
    ownerphone_2: typing_extensions.Annotated[typing.Optional[LinkData], FieldMetadata(alias="ownerphone2")] = None
    ownerssn: typing.Optional[LinkData] = None
    ownertitle: typing.Optional[LinkData] = None
    ozip: typing.Optional[LinkData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.additional_data import AdditionalData
from ..types.email import Email
from ..types.language import Language
from ..types.mfa_data import MfaData
from ..types.mfa_mode import MfaMode
from ..types.mfa_validation_code import MfaValidationCode
from ..types.name_user import NameUser
from ..types.org_scope import OrgScope
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_mfa_basic import PayabliApiResponseMfaBasic
from ..types.payabli_api_response_user_mfa import PayabliApiResponseUserMfa
from ..types.phone_number import PhoneNumber
from ..types.timezone import Timezone
from ..types.user_query_record import UserQueryRecord
from ..types.usr_access import UsrAccess
from ..types.usr_status import UsrStatus
from .raw_client import AsyncRawUserClient, RawUserClient
from .types.add_user_response import AddUserResponse
from .types.auth_reset_user_response import AuthResetUserResponse
from .types.change_psw_user_response import ChangePswUserResponse
from .types.delete_user_response import DeleteUserResponse
from .types.edit_mfa_user_response import EditMfaUserResponse
from .types.logout_user_response import LogoutUserResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUserClient
        """
        return self._raw_client

    def add_user(
        self,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddUserResponse:
        """
        Parameters
        ----------
        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.add_user()
        """
        _response = self._raw_client.add_user(
            access=access,
            additional_data=additional_data,
            email=email,
            language=language,
            mfa_data=mfa_data,
            name=name,
            phone=phone,
            pwd=pwd,
            scope=scope,
            time_zone=time_zone,
            usr_status=usr_status,
            request_options=request_options,
        )
        return _response.data

    def auth_refresh_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseUserMfa:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseUserMfa
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.auth_refresh_user()
        """
        _response = self._raw_client.auth_refresh_user(request_options=request_options)
        return _response.data

    def auth_reset_user(
        self,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthResetUserResponse:
        """
        Parameters
        ----------
        email : typing.Optional[Email]
            The user's email address.

        entry : typing.Optional[str]
            Identifier for entrypoint originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResetUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.auth_reset_user()
        """
        _response = self._raw_client.auth_reset_user(
            email=email, entry=entry, entry_type=entry_type, request_options=request_options
        )
        return _response.data

    def auth_user(
        self,
        provider: str,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        psw: typing.Optional[str] = OMIT,
        user_id: typing.Optional[int] = OMIT,
        user_token_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseMfaBasic:
        """
        This endpoint requires an application API token.

        Parameters
        ----------
        provider : str
            Auth provider. This fields is optional and defaults to null for the built-in provider.

        email : typing.Optional[Email]

        entry : typing.Optional[str]
            Identifier for entry point originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        psw : typing.Optional[str]

        user_id : typing.Optional[int]

        user_token_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseMfaBasic
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.auth_user(
            provider="provider",
        )
        """
        _response = self._raw_client.auth_user(
            provider,
            email=email,
            entry=entry,
            entry_type=entry_type,
            psw=psw,
            user_id=user_id,
            user_token_id=user_token_id,
            request_options=request_options,
        )
        return _response.data

    def change_psw_user(
        self, *, psw: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> ChangePswUserResponse:
        """
        Parameters
        ----------
        psw : typing.Optional[str]
            New User password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePswUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.change_psw_user()
        """
        _response = self._raw_client.change_psw_user(psw=psw, request_options=request_options)
        return _response.data

    def delete_user(
        self, user_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteUserResponse:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.delete_user(
            user_id=1000000,
        )
        """
        _response = self._raw_client.delete_user(user_id, request_options=request_options)
        return _response.data

    def edit_mfa_user(
        self,
        user_id: int,
        *,
        mfa: typing.Optional[bool] = OMIT,
        mfa_mode: typing.Optional[MfaMode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditMfaUserResponse:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        mfa : typing.Optional[bool]

        mfa_mode : typing.Optional[MfaMode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditMfaUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.edit_mfa_user(
            user_id=1000000,
        )
        """
        _response = self._raw_client.edit_mfa_user(user_id, mfa=mfa, mfa_mode=mfa_mode, request_options=request_options)
        return _response.data

    def edit_user(
        self,
        user_id: int,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.edit_user(
            user_id=1000000,
        )
        """
        _response = self._raw_client.edit_user(
            user_id,
            access=access,
            additional_data=additional_data,
            email=email,
            language=language,
            mfa_data=mfa_data,
            name=name,
            phone=phone,
            pwd=pwd,
            scope=scope,
            time_zone=time_zone,
            usr_status=usr_status,
            request_options=request_options,
        )
        return _response.data

    def get_user(
        self,
        user_id: int,
        *,
        entry: typing.Optional[str] = None,
        level: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserQueryRecord:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        entry : typing.Optional[str]
            The entrypoint identifier.

        level : typing.Optional[int]
            Entry level: 0 - partner, 2 - paypoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.get_user(
            user_id=1000000,
            entry="478ae1234",
        )
        """
        _response = self._raw_client.get_user(user_id, entry=entry, level=level, request_options=request_options)
        return _response.data

    def logout_user(self, *, request_options: typing.Optional[RequestOptions] = None) -> LogoutUserResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LogoutUserResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.logout_user()
        """
        _response = self._raw_client.logout_user(request_options=request_options)
        return _response.data

    def resend_mfa_code(
        self, entry: str, entry_type: int, usrname: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseMfaBasic:
        """
        Parameters
        ----------
        entry : str


        entry_type : int


        usrname : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseMfaBasic
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.resend_mfa_code(
            entry="Entry",
            entry_type=1,
            usrname="usrname",
        )
        """
        _response = self._raw_client.resend_mfa_code(entry, entry_type, usrname, request_options=request_options)
        return _response.data

    def validate_mfa_user(
        self,
        *,
        mfa_code: typing.Optional[str] = OMIT,
        mfa_validation_code: typing.Optional[MfaValidationCode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseUserMfa:
        """
        Parameters
        ----------
        mfa_code : typing.Optional[str]

        mfa_validation_code : typing.Optional[MfaValidationCode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseUserMfa
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.user.validate_mfa_user()
        """
        _response = self._raw_client.validate_mfa_user(
            mfa_code=mfa_code, mfa_validation_code=mfa_validation_code, request_options=request_options
        )
        return _response.data


class AsyncUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUserClient
        """
        return self._raw_client

    async def add_user(
        self,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddUserResponse:
        """
        Parameters
        ----------
        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.add_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.add_user(
            access=access,
            additional_data=additional_data,
            email=email,
            language=language,
            mfa_data=mfa_data,
            name=name,
            phone=phone,
            pwd=pwd,
            scope=scope,
            time_zone=time_zone,
            usr_status=usr_status,
            request_options=request_options,
        )
        return _response.data

    async def auth_refresh_user(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseUserMfa:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseUserMfa
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.auth_refresh_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.auth_refresh_user(request_options=request_options)
        return _response.data

    async def auth_reset_user(
        self,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthResetUserResponse:
        """
        Parameters
        ----------
        email : typing.Optional[Email]
            The user's email address.

        entry : typing.Optional[str]
            Identifier for entrypoint originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthResetUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.auth_reset_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.auth_reset_user(
            email=email, entry=entry, entry_type=entry_type, request_options=request_options
        )
        return _response.data

    async def auth_user(
        self,
        provider: str,
        *,
        email: typing.Optional[Email] = OMIT,
        entry: typing.Optional[str] = OMIT,
        entry_type: typing.Optional[int] = OMIT,
        psw: typing.Optional[str] = OMIT,
        user_id: typing.Optional[int] = OMIT,
        user_token_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseMfaBasic:
        """
        This endpoint requires an application API token.

        Parameters
        ----------
        provider : str
            Auth provider. This fields is optional and defaults to null for the built-in provider.

        email : typing.Optional[Email]

        entry : typing.Optional[str]
            Identifier for entry point originating the request (used by front-end apps)

        entry_type : typing.Optional[int]
            Type of entry identifier: 0 - partner, 2 - paypoint. This is used by front-end apps, required if an Entry is indicated.

        psw : typing.Optional[str]

        user_id : typing.Optional[int]

        user_token_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseMfaBasic
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.auth_user(
                provider="provider",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.auth_user(
            provider,
            email=email,
            entry=entry,
            entry_type=entry_type,
            psw=psw,
            user_id=user_id,
            user_token_id=user_token_id,
            request_options=request_options,
        )
        return _response.data

    async def change_psw_user(
        self, *, psw: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> ChangePswUserResponse:
        """
        Parameters
        ----------
        psw : typing.Optional[str]
            New User password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePswUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.change_psw_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.change_psw_user(psw=psw, request_options=request_options)
        return _response.data

    async def delete_user(
        self, user_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteUserResponse:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.delete_user(
                user_id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_user(user_id, request_options=request_options)
        return _response.data

    async def edit_mfa_user(
        self,
        user_id: int,
        *,
        mfa: typing.Optional[bool] = OMIT,
        mfa_mode: typing.Optional[MfaMode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditMfaUserResponse:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        mfa : typing.Optional[bool]

        mfa_mode : typing.Optional[MfaMode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditMfaUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.edit_mfa_user(
                user_id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.edit_mfa_user(
            user_id, mfa=mfa, mfa_mode=mfa_mode, request_options=request_options
        )
        return _response.data

    async def edit_user(
        self,
        user_id: int,
        *,
        access: typing.Optional[typing.Sequence[UsrAccess]] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        email: typing.Optional[Email] = OMIT,
        language: typing.Optional[Language] = OMIT,
        mfa_data: typing.Optional[MfaData] = OMIT,
        name: typing.Optional[NameUser] = OMIT,
        phone: typing.Optional[PhoneNumber] = OMIT,
        pwd: typing.Optional[str] = OMIT,
        scope: typing.Optional[typing.Sequence[OrgScope]] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        usr_status: typing.Optional[UsrStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponse:
        """
        Parameters
        ----------
        user_id : int
            User Identifier

        access : typing.Optional[typing.Sequence[UsrAccess]]

        additional_data : typing.Optional[AdditionalData]

        email : typing.Optional[Email]
            The user's email address.

        language : typing.Optional[Language]

        mfa_data : typing.Optional[MfaData]

        name : typing.Optional[NameUser]

        phone : typing.Optional[PhoneNumber]
            The user's phone number.

        pwd : typing.Optional[str]

        scope : typing.Optional[typing.Sequence[OrgScope]]

        time_zone : typing.Optional[Timezone]

        usr_status : typing.Optional[UsrStatus]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.edit_user(
                user_id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.edit_user(
            user_id,
            access=access,
            additional_data=additional_data,
            email=email,
            language=language,
            mfa_data=mfa_data,
            name=name,
            phone=phone,
            pwd=pwd,
            scope=scope,
            time_zone=time_zone,
            usr_status=usr_status,
            request_options=request_options,
        )
        return _response.data

    async def get_user(
        self,
        user_id: int,
        *,
        entry: typing.Optional[str] = None,
        level: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserQueryRecord:
        """
        Parameters
        ----------
        user_id : int
            The Payabli-generated `userId` value.

        entry : typing.Optional[str]
            The entrypoint identifier.

        level : typing.Optional[int]
            Entry level: 0 - partner, 2 - paypoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_user(
                user_id=1000000,
                entry="478ae1234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user(user_id, entry=entry, level=level, request_options=request_options)
        return _response.data

    async def logout_user(self, *, request_options: typing.Optional[RequestOptions] = None) -> LogoutUserResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LogoutUserResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.logout_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.logout_user(request_options=request_options)
        return _response.data

    async def resend_mfa_code(
        self, entry: str, entry_type: int, usrname: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseMfaBasic:
        """
        Parameters
        ----------
        entry : str


        entry_type : int


        usrname : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseMfaBasic
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.resend_mfa_code(
                entry="Entry",
                entry_type=1,
                usrname="usrname",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.resend_mfa_code(entry, entry_type, usrname, request_options=request_options)
        return _response.data

    async def validate_mfa_user(
        self,
        *,
        mfa_code: typing.Optional[str] = OMIT,
        mfa_validation_code: typing.Optional[MfaValidationCode] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseUserMfa:
        """
        Parameters
        ----------
        mfa_code : typing.Optional[str]

        mfa_validation_code : typing.Optional[MfaValidationCode]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseUserMfa
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.validate_mfa_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.validate_mfa_user(
            mfa_code=mfa_code, mfa_validation_code=mfa_validation_code, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

Whenrefunded = typing.Union[
    typing.Literal["Exchange Only", "No Refund or Exchange", "More than 30 days", "30 Days or Less"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.attachments import Attachments
from ..types.email import Email
from ..types.idempotency_key import IdempotencyKey
from ..types.payabli_api_response import PayabliApiResponse
from .types.add_response_response import AddResponseResponse
from .types.chargeback_query_records import ChargebackQueryRecords

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawChargeBacksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_response(
        self,
        id: int,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        attachments: typing.Optional[Attachments] = OMIT,
        contact_email: typing.Optional[Email] = OMIT,
        contact_name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddResponseResponse]:
        """
        Add a response to a chargeback or ACH return.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record.

        idempotency_key : typing.Optional[IdempotencyKey]

        attachments : typing.Optional[Attachments]
            Array of attached files to response.

        contact_email : typing.Optional[Email]
            Email of response submitter.

        contact_name : typing.Optional[str]
            Name of response submitter

        notes : typing.Optional[str]
            Response notes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddResponseResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"ChargeBacks/response/{jsonable_encoder(id)}",
            method="POST",
            json={
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "contactEmail": contact_email,
                "contactName": contact_name,
                "notes": notes,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddResponseResponse,
                    parse_obj_as(
                        type_=AddResponseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_chargeback(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ChargebackQueryRecords]:
        """
        Retrieves a chargeback record and its details.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record. This is returned as `chargebackId` in the [RecievedChargeback](/developers/developer-guides/webhook-payloads#receivedChargeback) and [ReceivedAchReturn](/developers/developer-guides/webhook-payloads#receivedachreturn) webhook notifications.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChargebackQueryRecords]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"ChargeBacks/read/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChargebackQueryRecords,
                    parse_obj_as(
                        type_=ChargebackQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_chargeback_attachment(
        self, file_name: str, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[str]:
        """
        Parameters
        ----------
        file_name : str
            The chargeback attachment's file name.

        id : int
            The ID of chargeback or return record.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[str]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"ChargeBacks/getChargebackAttachments/{jsonable_encoder(id)}/{jsonable_encoder(file_name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=_response.text)  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawChargeBacksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_response(
        self,
        id: int,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        attachments: typing.Optional[Attachments] = OMIT,
        contact_email: typing.Optional[Email] = OMIT,
        contact_name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddResponseResponse]:
        """
        Add a response to a chargeback or ACH return.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record.

        idempotency_key : typing.Optional[IdempotencyKey]

        attachments : typing.Optional[Attachments]
            Array of attached files to response.

        contact_email : typing.Optional[Email]
            Email of response submitter.

        contact_name : typing.Optional[str]
            Name of response submitter

        notes : typing.Optional[str]
            Response notes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddResponseResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"ChargeBacks/response/{jsonable_encoder(id)}",
            method="POST",
            json={
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "contactEmail": contact_email,
                "contactName": contact_name,
                "notes": notes,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddResponseResponse,
                    parse_obj_as(
                        type_=AddResponseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_chargeback(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ChargebackQueryRecords]:
        """
        Retrieves a chargeback record and its details.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record. This is returned as `chargebackId` in the [RecievedChargeback](/developers/developer-guides/webhook-payloads#receivedChargeback) and [ReceivedAchReturn](/developers/developer-guides/webhook-payloads#receivedachreturn) webhook notifications.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChargebackQueryRecords]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"ChargeBacks/read/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChargebackQueryRecords,
                    parse_obj_as(
                        type_=ChargebackQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_chargeback_attachment(
        self, file_name: str, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[str]:
        """
        Parameters
        ----------
        file_name : str
            The chargeback attachment's file name.

        id : int
            The ID of chargeback or return record.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[str]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"ChargeBacks/getChargebackAttachments/{jsonable_encoder(id)}/{jsonable_encoder(file_name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=_response.text)  # type: ignore
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accounting_field import AccountingField
from .bill_item import BillItem
from .bill_query_record_2_bill_approvals_item import BillQueryRecord2BillApprovalsItem
from .bill_query_record_2_payment_method import BillQueryRecord2PaymentMethod
from .billstatus import Billstatus
from .created_at import CreatedAt
from .datenullable import Datenullable
from .datetime_nullable import DatetimeNullable
from .dbaname import Dbaname
from .external_paypoint_id import ExternalPaypointId
from .frequency import Frequency
from .general_events import GeneralEvents
from .legalname import Legalname
from .org_parent_name import OrgParentName
from .payment_id_string import PaymentIdString
from .terms import Terms
from .transaction_out_query_record import TransactionOutQueryRecord
from .vendor_out_data import VendorOutData


class BillQueryRecord2(UniversalBaseModel):
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="AccountingField1")
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="AccountingField2")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="AdditionalData")
    ] = pydantic.Field(default=None)
    """
    Additional data associated with the bill.
    """

    batch_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BatchNumber")] = (
        pydantic.Field(default=None)
    )
    """
    Batch number associated with the bill.
    """

    bill_approvals: typing_extensions.Annotated[
        typing.Optional[typing.List[BillQueryRecord2BillApprovalsItem]], FieldMetadata(alias="billApprovals")
    ] = None
    bill_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="BillDate")] = (
        pydantic.Field(default=None)
    )
    """
    Bill creation date in one of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    bill_events: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]], FieldMetadata(alias="billEvents")
    ] = pydantic.Field(default=None)
    """
    Events associated with the bill.
    """

    bill_items: typing_extensions.Annotated[
        typing.Optional[typing.List[BillItem]], FieldMetadata(alias="BillItems")
    ] = pydantic.Field(default=None)
    """
    Array of items included in the bill.
    """

    bill_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BillNumber")] = pydantic.Field(
        default=None
    )
    """
    Bill number.
    """

    comments: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Comments")] = pydantic.Field(
        default=None
    )
    """
    Additional comments on the bill.
    """

    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of when bill was created, in UTC.
    """

    discount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="Discount")] = pydantic.Field(
        default=None
    )
    """
    Discount amount applied to the bill.
    """

    documents_ref: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="DocumentsRef")] = (
        pydantic.Field(default=None)
    )
    """
    Reference to documents associated with the bill.
    """

    due_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="DueDate")] = (
        pydantic.Field(default=None)
    )
    """
    Bill due date in one of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    end_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="EndDate")] = (
        pydantic.Field(default=None)
    )
    """
    End date for the bill.
    """

    entity_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EntityID")] = pydantic.Field(
        default=None
    )
    """
    Entity identifier associated with the bill.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    frequency: typing_extensions.Annotated[typing.Optional[Frequency], FieldMetadata(alias="Frequency")] = (
        pydantic.Field(default=None)
    )
    """
    Frequency for scheduled bills. Applied only in `Mode` = 1.
    """

    id_bill: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="IdBill")] = pydantic.Field(
        default=None
    )
    """
    Identifier of the bill.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[DatetimeNullable], FieldMetadata(alias="LastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of when bill was last updated, in UTC.
    """

    lot_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="LotNumber")] = pydantic.Field(
        default=None
    )
    """
    Lot number associated with the bill.
    """

    mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Mode")] = pydantic.Field(default=None)
    """
    Bill mode: value `0` for single/one-time bills, `1` for scheduled bills.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="NetAmount")] = pydantic.Field(
        default=None
    )
    """
    Net amount of the bill.
    """

    parent_org_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ParentOrgId")] = (
        pydantic.Field(default=None)
    )
    """
    Parent organization identifier.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    payment_id: typing_extensions.Annotated[typing.Optional[PaymentIdString], FieldMetadata(alias="PaymentId")] = None
    payment_method: typing_extensions.Annotated[
        typing.Optional[BillQueryRecord2PaymentMethod], FieldMetadata(alias="PaymentMethod")
    ] = pydantic.Field(default=None)
    """
    Preferred payment method used.
    """

    paylink_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paylinkId")] = pydantic.Field(
        default=None
    )
    """
    Paylink identifier associated with the bill.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field(default=None)
    )
    """
    Entry name of the paypoint.
    """

    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's legal name.
    """

    source: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Source")] = pydantic.Field(
        default=None
    )
    """
    Source of the bill.
    """

    status: typing_extensions.Annotated[typing.Optional[Billstatus], FieldMetadata(alias="Status")] = None
    terms: typing_extensions.Annotated[typing.Optional[Terms], FieldMetadata(alias="Terms")] = pydantic.Field(
        default=None
    )
    """
    The payment terms for invoice. If no terms were defined initially, then response data for this field will default to `N30`.
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount of the bill including taxes and fees.
    """

    transaction: typing_extensions.Annotated[
        typing.Optional[TransactionOutQueryRecord], FieldMetadata(alias="Transaction")
    ] = pydantic.Field(default=None)
    """
    MoneyOut transaction associated to the bill.
    """

    vendor: typing_extensions.Annotated[typing.Optional[VendorOutData], FieldMetadata(alias="Vendor")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
